# Generated by Django 3.2.11 on 2022-03-12 16:29

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_screening", "0037_auto_20220312_0339"),
    ]

    operations = [
        migrations.RenameField(
            model_name="historicalscreeningpartone",
            old_name="repeat_fbg_performed",
            new_name="fbg2_performed",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartone",
            old_name="ifg2_quantifier",
            new_name="fbg2_quantifier",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartone",
            old_name="ifg2_value",
            new_name="fbg2_value",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartone",
            old_name="ifg_datetime",
            new_name="fbg_datetime",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartone",
            old_name="ifg_quantifier",
            new_name="fbg_quantifier",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartone",
            old_name="ifg_units",
            new_name="fbg_units",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartone",
            old_name="ifg_value",
            new_name="fbg_value",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartone",
            old_name="repeat_ogtt_performed",
            new_name="ogtt2_performed",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartthree",
            old_name="ifg2_datetime",
            new_name="fbg2_datetime",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartthree",
            old_name="repeat_fbg_performed",
            new_name="fbg2_performed",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartthree",
            old_name="ifg2_quantifier",
            new_name="fbg2_quantifier",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartthree",
            old_name="ifg2_value",
            new_name="fbg2_value",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartthree",
            old_name="ifg_datetime",
            new_name="fbg_datetime",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartthree",
            old_name="ifg_quantifier",
            new_name="fbg_quantifier",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartthree",
            old_name="ifg_units",
            new_name="fbg_units",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartthree",
            old_name="ifg_value",
            new_name="fbg_value",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartthree",
            old_name="repeat_ogtt_performed",
            new_name="ogtt2_performed",
        ),
        migrations.RenameField(
            model_name="historicalscreeningparttwo",
            old_name="ifg2_datetime",
            new_name="fbg2_datetime",
        ),
        migrations.RenameField(
            model_name="historicalscreeningparttwo",
            old_name="repeat_fbg_performed",
            new_name="fbg2_performed",
        ),
        migrations.RenameField(
            model_name="historicalscreeningparttwo",
            old_name="ifg2_quantifier",
            new_name="fbg2_quantifier",
        ),
        migrations.RenameField(
            model_name="historicalscreeningparttwo",
            old_name="ifg2_value",
            new_name="fbg2_value",
        ),
        migrations.RenameField(
            model_name="historicalscreeningparttwo",
            old_name="ifg_datetime",
            new_name="fbg_datetime",
        ),
        migrations.RenameField(
            model_name="historicalscreeningparttwo",
            old_name="ifg_quantifier",
            new_name="fbg_quantifier",
        ),
        migrations.RenameField(
            model_name="historicalscreeningparttwo",
            old_name="ifg_units",
            new_name="fbg_units",
        ),
        migrations.RenameField(
            model_name="historicalscreeningparttwo",
            old_name="ifg_value",
            new_name="fbg_value",
        ),
        migrations.RenameField(
            model_name="historicalscreeningparttwo",
            old_name="repeat_ogtt_performed",
            new_name="ogtt2_performed",
        ),
        migrations.RenameField(
            model_name="historicalsubjectscreening",
            old_name="ifg2_datetime",
            new_name="fbg2_datetime",
        ),
        migrations.RenameField(
            model_name="historicalsubjectscreening",
            old_name="repeat_fbg_performed",
            new_name="fbg2_performed",
        ),
        migrations.RenameField(
            model_name="historicalsubjectscreening",
            old_name="ifg2_quantifier",
            new_name="fbg2_quantifier",
        ),
        migrations.RenameField(
            model_name="historicalsubjectscreening",
            old_name="ifg2_value",
            new_name="fbg2_value",
        ),
        migrations.RenameField(
            model_name="historicalsubjectscreening",
            old_name="ifg_datetime",
            new_name="fbg_datetime",
        ),
        migrations.RenameField(
            model_name="historicalsubjectscreening",
            old_name="ifg_quantifier",
            new_name="fbg_quantifier",
        ),
        migrations.RenameField(
            model_name="historicalsubjectscreening",
            old_name="ifg_units",
            new_name="fbg_units",
        ),
        migrations.RenameField(
            model_name="historicalsubjectscreening",
            old_name="ifg_value",
            new_name="fbg_value",
        ),
        migrations.RenameField(
            model_name="historicalsubjectscreening",
            old_name="repeat_ogtt_performed",
            new_name="ogtt2_performed",
        ),
        migrations.RenameField(
            model_name="subjectscreening",
            old_name="ifg2_datetime",
            new_name="fbg2_datetime",
        ),
        migrations.RenameField(
            model_name="subjectscreening",
            old_name="repeat_fbg_performed",
            new_name="fbg2_performed",
        ),
        migrations.RenameField(
            model_name="subjectscreening",
            old_name="ifg2_quantifier",
            new_name="fbg2_quantifier",
        ),
        migrations.RenameField(
            model_name="subjectscreening",
            old_name="ifg2_value",
            new_name="fbg2_value",
        ),
        migrations.RenameField(
            model_name="subjectscreening",
            old_name="ifg_datetime",
            new_name="fbg_datetime",
        ),
        migrations.RenameField(
            model_name="subjectscreening",
            old_name="ifg_quantifier",
            new_name="fbg_quantifier",
        ),
        migrations.RenameField(
            model_name="subjectscreening",
            old_name="ifg_units",
            new_name="fbg_units",
        ),
        migrations.RenameField(
            model_name="subjectscreening",
            old_name="ifg_value",
            new_name="fbg_value",
        ),
        migrations.RenameField(
            model_name="subjectscreening",
            old_name="repeat_ogtt_performed",
            new_name="ogtt2_performed",
        ),
        migrations.RenameField(
            model_name="historicalicpreferral",
            old_name="ifg_value",
            new_name="fbg_value",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartone",
            old_name="converted_ifg2_value",
            new_name="converted_fbg2_value",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartone",
            old_name="converted_ifg_value",
            new_name="converted_fbg_value",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartone",
            old_name="ifg2_datetime",
            new_name="fbg2_datetime",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartone",
            old_name="ifg2_units",
            new_name="fbg2_units",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartthree",
            old_name="converted_ifg2_value",
            new_name="converted_fbg2_value",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartthree",
            old_name="converted_ifg_value",
            new_name="converted_fbg_value",
        ),
        migrations.RenameField(
            model_name="historicalscreeningpartthree",
            old_name="ifg2_units",
            new_name="fbg2_units",
        ),
        migrations.RenameField(
            model_name="historicalscreeningparttwo",
            old_name="converted_ifg2_value",
            new_name="converted_fbg2_value",
        ),
        migrations.RenameField(
            model_name="historicalscreeningparttwo",
            old_name="converted_ifg_value",
            new_name="converted_fbg_value",
        ),
        migrations.RenameField(
            model_name="historicalscreeningparttwo",
            old_name="ifg2_units",
            new_name="fbg2_units",
        ),
        migrations.RenameField(
            model_name="historicalsubjectscreening",
            old_name="converted_ifg2_value",
            new_name="converted_fbg2_value",
        ),
        migrations.RenameField(
            model_name="historicalsubjectscreening",
            old_name="converted_ifg_value",
            new_name="converted_fbg_value",
        ),
        migrations.RenameField(
            model_name="historicalsubjectscreening",
            old_name="ifg2_units",
            new_name="fbg2_units",
        ),
        migrations.RenameField(
            model_name="icpreferral",
            old_name="ifg_value",
            new_name="fbg_value",
        ),
        migrations.RenameField(
            model_name="subjectscreening",
            old_name="converted_ifg2_value",
            new_name="converted_fbg2_value",
        ),
        migrations.RenameField(
            model_name="subjectscreening",
            old_name="converted_ifg_value",
            new_name="converted_fbg_value",
        ),
        migrations.RenameField(
            model_name="subjectscreening",
            old_name="ifg2_units",
            new_name="fbg2_units",
        ),
    ]



# Generated by Django 3.2.11 on 2022-04-03 09:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_screening", "0040_auto_20220316_2147"),
    ]

    operations = [
        migrations.AddField(
            model_name="historicalscreeningpartone",
            name="vl_undetectable",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="Yes",
                max_length=15,
                verbose_name="Does the patient have a viral load measure of < 400 copies/ml taken within the last 6 months",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="historicalscreeningpartthree",
            name="vl_undetectable",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="Yes",
                max_length=15,
                verbose_name="Does the patient have a viral load measure of < 400 copies/ml taken within the last 6 months",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="historicalscreeningparttwo",
            name="vl_undetectable",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="Yes",
                max_length=15,
                verbose_name="Does the patient have a viral load measure of < 400 copies/ml taken within the last 6 months",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="historicalsubjectscreening",
            name="vl_undetectable",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="Yes",
                max_length=15,
                verbose_name="Does the patient have a viral load measure of < 400 copies/ml taken within the last 6 months",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="subjectscreening",
            name="vl_undetectable",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="Yes",
                max_length=15,
                verbose_name="Does the patient have a viral load measure of < 400 copies/ml taken within the last 6 months",
            ),
            preserve_default=False,
        ),
    ]



# Generated by Django 3.2.13 on 2022-08-26 00:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_ae", "0012_auto_20220826_0258"),
    ]

    operations = [
        migrations.AlterField(
            model_name="aefollowup",
            name="tracking_identifier",
            field=models.CharField(max_length=32, null=True, unique=True),
        ),
        migrations.AlterField(
            model_name="aeinitial",
            name="tracking_identifier",
            field=models.CharField(max_length=32, null=True, unique=True),
        ),
        migrations.AlterField(
            model_name="aesusar",
            name="tracking_identifier",
            field=models.CharField(max_length=32, null=True, unique=True),
        ),
        migrations.AlterField(
            model_name="aetmg",
            name="tracking_identifier",
            field=models.CharField(max_length=32, null=True, unique=True),
        ),
        migrations.AlterField(
            model_name="deathreport",
            name="tracking_identifier",
            field=models.CharField(max_length=32, null=True, unique=True),
        ),
        migrations.AlterField(
            model_name="deathreporttmg",
            name="tracking_identifier",
            field=models.CharField(max_length=32, null=True, unique=True),
        ),
        migrations.AlterField(
            model_name="historicalaefollowup",
            name="tracking_identifier",
            field=models.CharField(db_index=True, max_length=32, null=True),
        ),
        migrations.AlterField(
            model_name="historicalaeinitial",
            name="tracking_identifier",
            field=models.CharField(db_index=True, max_length=32, null=True),
        ),
        migrations.AlterField(
            model_name="historicalaesusar",
            name="tracking_identifier",
            field=models.CharField(db_index=True, max_length=32, null=True),
        ),
        migrations.AlterField(
            model_name="historicalaetmg",
            name="tracking_identifier",
            field=models.CharField(db_index=True, max_length=32, null=True),
        ),
        migrations.AlterField(
            model_name="historicaldeathreport",
            name="tracking_identifier",
            field=models.CharField(db_index=True, max_length=32, null=True),
        ),
        migrations.AlterField(
            model_name="historicaldeathreporttmg",
            name="tracking_identifier",
            field=models.CharField(db_index=True, max_length=32, null=True),
        ),
        migrations.AlterField(
            model_name="historicaldeathreporttmgsecond",
            name="tracking_identifier",
            field=models.CharField(db_index=True, max_length=32, null=True),
        ),
    ]



from django import forms
from edc_crf.crf_form_validator import CrfFormValidator
from edc_crf.modelform_mixins import CrfModelFormMixin

from ..models import FollowupVitals


class FollowupVitalsFormValidator(CrfFormValidator):
    pass


class FollowupVitalsForm(CrfModelFormMixin, forms.ModelForm):

    form_validator_cls = FollowupVitalsFormValidator

    class Meta:
        model = FollowupVitals
        fields = "__all__"



from django.contrib import admin
from django.db import models
from edc_constants.choices import POS_NEG_NA, YES_NO, YES_NO_NA
from edc_constants.constants import NO, NOT_APPLICABLE

from ..choices import YES_NO_NOT_ELIGIBLE, YES_PENDING_NA
from ..constants import PREG_YES_NO_NA


class PartTwoFieldsModelMixin(models.Model):

    part_two_report_datetime = models.DateTimeField(
        verbose_name="Part 2 report date and time",
        null=True,
        blank=False,
        help_text="Date and time of report.",
    )

    urine_bhcg_performed = models.CharField(
        verbose_name="Was a Urine or serum βhCG performed?",
        max_length=15,
        choices=PREG_YES_NO_NA,
        default=NOT_APPLICABLE,
        help_text="(Pregnancy test)",
    )

    urine_bhcg_value = models.CharField(
        verbose_name="Urine or serum βhCG result",
        max_length=15,
        choices=POS_NEG_NA,
        default=NOT_APPLICABLE,
    )

    urine_bhcg_date = models.DateField(
        verbose_name="Urine or serum βhCG date", blank=True, null=True
    )

    congestive_heart_failure = models.CharField(
        verbose_name=(
            "Does the patient have congestive heart failure requiring pharmacologic therapy"
        ),
        max_length=15,
        choices=YES_NO,
        null=True,
        blank=False,
    )

    liver_disease = models.CharField(
        verbose_name="Is there clinical evidence of liver disease",
        max_length=15,
        choices=YES_NO,
        null=True,
        blank=False,
        help_text=(
            "Evidence of chronic liver disease: Jaundice, pruritus, "
            "hepatomegaly, ascites, spider naevi and flapping tremors."
        ),
    )

    alcoholism = models.CharField(
        verbose_name=(
            "Does the patient have any evidence of alcoholism or acute alcohol intoxication"
        ),
        max_length=15,
        choices=YES_NO,
        null=True,
        blank=False,
        help_text=(
            "Evidence of alcoholism or acute alcohol intoxication: "
            "flushing, amnesia, mental confusion, nausea or vomiting, "
            "slurred speech, dehydration, dry skin and brittle hair."
        ),
    )

    acute_metabolic_acidosis = models.CharField(
        verbose_name=(
            "Does the patient have any signs or symptoms of acute metabolic acidosis"
        ),
        max_length=15,
        choices=YES_NO,
        null=True,
        blank=False,
        help_text="lactic acidosis and/or diabetic ketoacidosis",
    )
    renal_function_condition = models.CharField(
        verbose_name=(
            "Does the patient have any acute condition which can alter renal "
            "function including: dehydration, severe infection or shock"
        ),
        max_length=15,
        choices=YES_NO,
        null=True,
        blank=False,
    )

    tissue_hypoxia_condition = models.CharField(
        verbose_name=(
            "Does the patient have any acute condition which can cause tissue hypoxia"
        ),
        max_length=15,
        choices=YES_NO,
        null=True,
        blank=False,
        help_text=(
            "Including: decompensated heart failure, respiratory failure, "
            "recent myocardial infarction or shock"
        ),
    )

    acute_condition = models.CharField(
        verbose_name=(
            "Does the patient have any acute condition requiring "
            "immediate hospital care/admission"
        ),
        max_length=15,
        choices=YES_NO,
        null=True,
        blank=False,
    )

    metformin_sensitivity = models.CharField(
        verbose_name=(
            "Does the patient have any known hypersensitivity to metformin "
            "or any excipients associated with its preparation"
        ),
        max_length=15,
        choices=YES_NO,
        null=True,
        blank=False,
        help_text=(
            "For example: Magnesium stearate, sodium carboxymethylcellulose, hypromellose"
        ),
    )

    has_dm = models.CharField(
        verbose_name="Is the patient known to have diabetes?",
        max_length=15,
        choices=YES_NO,
        null=True,
        blank=False,
    )

    on_dm_medication = models.CharField(
        verbose_name="Is the patient known to be taking anti-diabetic medications?",
        max_length=15,
        choices=YES_NO,
        null=True,
        blank=False,
    )

    already_fasted = models.CharField(
        verbose_name="Has the patient come to the clinic today already fasted?",
        max_length=15,
        choices=YES_NO_NOT_ELIGIBLE,
        default=NO,
    )

    agree_to_p3 = models.CharField(
        verbose_name=(
            "Has the patient agreed to complete/return for the second stage of the screening?"
        ),
        max_length=15,
        choices=YES_NO_NOT_ELIGIBLE,
        default=NOT_APPLICABLE,
        help_text=(
            "If patient is not continuing to the second stage today, advised to "
            "fast and provide a return appointment date below"
        ),
    )

    advised_to_fast = models.CharField(
        verbose_name=(
            "Has the patient been advised to return FASTED for the second "
            "stage of the screening?"
        ),
        max_length=15,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
        help_text="Not applicable if continuing to the second stage today",
    )

    appt_datetime = models.DateTimeField(
        verbose_name="Appointment date for second stage of screening (P3)",
        null=True,
        blank=True,
        help_text="Leave blank if continuing to the second stage today",
    )

    p3_ltfu = models.CharField(
        max_length=15,
        verbose_name="Consider the patient 'lost to screening' for now?",
        choices=YES_PENDING_NA,
        default=NOT_APPLICABLE,
        help_text=(
            "Only applicable if the patient missed the appointment for the second stage "
            "of screening (P3), several attempts have been made to contact the patient, "
            "and the patient has not started P3. See above"
        ),
    )

    p3_ltfu_date = models.DateField(
        verbose_name="Date decision made",
        null=True,
        blank=True,
        help_text="Must be after the appointment date for the second stage of screening (P3)",
    )

    p3_ltfu_comment = models.TextField(
        verbose_name="Provide any additional comments on this decision (or leave blank)",
        null=True,
        blank=True,
    )

    @admin.display(description="P3 appointment", ordering="appt_datetime")
    def p3_appt(self):
        return self.appt_datetime

    class Meta:
        abstract = True



# Generated by Django 4.0.5 on 2022-07-03 09:35

from django.db import migrations, models
import django.db.models.deletion
import edc_model.validators.date


class Migration(migrations.Migration):

    dependencies = [
        ("meta_lists", "0012_auto_20210728_1809"),
        ("meta_prn", "0036_remove_endofstudy_meta_prn_en_id_a50384_idx_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="endofstudy",
            name="delivery_date",
            field=models.DateField(
                blank=True,
                help_text="A delivery CRF must be on file",
                null=True,
                validators=[edc_model.validators.date.date_not_future],
                verbose_name="Date of delivery, if applicable",
            ),
        ),
        migrations.AddField(
            model_name="endofstudy",
            name="pregnancy_date",
            field=models.DateField(
                blank=True,
                help_text="A UPT CRF must be on file. Use UPT date or, if UPT not done, use report date on last UPT CRF.",
                null=True,
                validators=[edc_model.validators.date.date_not_future],
                verbose_name="Date pregnancy reported/UPT, if applicable",
            ),
        ),
        migrations.AddField(
            model_name="historicalendofstudy",
            name="delivery_date",
            field=models.DateField(
                blank=True,
                help_text="A delivery CRF must be on file",
                null=True,
                validators=[edc_model.validators.date.date_not_future],
                verbose_name="Date of delivery, if applicable",
            ),
        ),
        migrations.AddField(
            model_name="historicalendofstudy",
            name="pregnancy_date",
            field=models.DateField(
                blank=True,
                help_text="A UPT CRF must be on file. Use UPT date or, if UPT not done, use report date on last UPT CRF.",
                null=True,
                validators=[edc_model.validators.date.date_not_future],
                verbose_name="Date pregnancy reported/UPT, if applicable",
            ),
        ),
        migrations.AlterField(
            model_name="endofstudy",
            name="clinical_withdrawal_reason",
            field=models.CharField(
                blank=True,
                choices=[
                    ("diabetes", "Patient developed diabetes"),
                    ("kidney_disease", "Development of chronic kidney disease"),
                    ("liver_disease", "Development of chronic liver disease"),
                    ("toxicity", "Patient experienced an unacceptable toxicity"),
                    (
                        "intercurrent_illness",
                        "Intercurrent illness which prevents further treatment",
                    ),
                    ("investigator_decision", "Investigator decision"),
                    (
                        "OTHER",
                        "Other condition that justifies the discontinuation of treatment in the clinician’s opinion (specify below)",
                    ),
                ],
                max_length=25,
                null=True,
                verbose_name="If withdrawn on CLINICAL grounds, please specify PRIMARY reason...",
            ),
        ),
        migrations.AlterField(
            model_name="endofstudy",
            name="death_date",
            field=models.DateField(
                blank=True,
                help_text="A Death report must be on file",
                null=True,
                validators=[edc_model.validators.date.date_not_future],
                verbose_name="Date of death, if applicable",
            ),
        ),
        migrations.AlterField(
            model_name="endofstudy",
            name="offstudy_reason",
            field=models.ForeignKey(
                limit_choices_to={
                    "name__in": [
                        "completed_followup",
                        "delivery",
                        "pregnancy",
                        "clinical_withdrawal",
                        "clinical_endpoint",
                        "LTFU",
                        "dead",
                        "withdrawal",
                        "late_exclusion",
                        "transferred",
                        "OTHER",
                    ]
                },
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to="meta_lists.offstudyreasons",
                verbose_name="Reason patient was terminated from the study",
            ),
        ),
        migrations.AlterField(
            model_name="historicalendofstudy",
            name="clinical_withdrawal_reason",
            field=models.CharField(
                blank=True,
                choices=[
                    ("diabetes", "Patient developed diabetes"),
                    ("kidney_disease", "Development of chronic kidney disease"),
                    ("liver_disease", "Development of chronic liver disease"),
                    ("toxicity", "Patient experienced an unacceptable toxicity"),
                    (
                        "intercurrent_illness",
                        "Intercurrent illness which prevents further treatment",
                    ),
                    ("investigator_decision", "Investigator decision"),
                    (
                        "OTHER",
                        "Other condition that justifies the discontinuation of treatment in the clinician’s opinion (specify below)",
                    ),
                ],
                max_length=25,
                null=True,
                verbose_name="If withdrawn on CLINICAL grounds, please specify PRIMARY reason...",
            ),
        ),
        migrations.AlterField(
            model_name="historicalendofstudy",
            name="death_date",
            field=models.DateField(
                blank=True,
                help_text="A Death report must be on file",
                null=True,
                validators=[edc_model.validators.date.date_not_future],
                verbose_name="Date of death, if applicable",
            ),
        ),
        migrations.AlterField(
            model_name="historicalendofstudy",
            name="offstudy_reason",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                limit_choices_to={
                    "name__in": [
                        "completed_followup",
                        "delivery",
                        "pregnancy",
                        "clinical_withdrawal",
                        "clinical_endpoint",
                        "LTFU",
                        "dead",
                        "withdrawal",
                        "late_exclusion",
                        "transferred",
                        "OTHER",
                    ]
                },
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="meta_lists.offstudyreasons",
                verbose_name="Reason patient was terminated from the study",
            ),
        ),
    ]



from django import forms
from edc_action_item.forms import ActionItemCrfFormMixin
from edc_constants.constants import NONE, OTHER, YES
from edc_crf.crf_form_validator import CrfFormValidator
from edc_crf.modelform_mixins import CrfModelFormMixin

from ..models import FollowupExamination


class FollowupExaminationFormValidator(CrfFormValidator):
    def clean(self):

        self.validate_all_symptoms()

        self.validate_other_problems()

        self.validate_hospitalizations()

        self.validate_hiv_medications()

        self.required_if(YES, field="referral", field_required="referral_reason")

    def validate_all_symptoms(self):

        self.m2m_single_selection_if(NONE, m2m_field="symptoms")
        self.m2m_other_not_specify(NONE, m2m_field="symptoms", field_other="symptoms_detail")
        self.m2m_single_selection_if(NONE, m2m_field="symptoms_g3")
        self.m2m_single_selection_if(NONE, m2m_field="symptoms_g4")

        # G3 and G4 selections, if specified, should come from the original symptoms list
        symptoms = []
        if self.cleaned_data.get("symptoms"):
            symptoms = [
                obj.name for obj in self.cleaned_data.get("symptoms") if obj.name != NONE
            ]
        symptoms_g3 = []
        if self.cleaned_data.get("symptoms_g3"):
            symptoms_g3 = [obj.name for obj in self.cleaned_data.get("symptoms_g3")]
        symptoms_g4 = []
        if self.cleaned_data.get("symptoms_g4"):
            symptoms_g4 = [obj.name for obj in self.cleaned_data.get("symptoms_g4")]
        if symptoms_g3 != [NONE] and [x for x in symptoms_g3 if x not in symptoms]:
            raise forms.ValidationError(
                {"symptoms_g3": "Invalid selection. Must be from above list of symptoms"}
            )
        if symptoms_g4 != [NONE] and [x for x in symptoms_g4 if x not in symptoms]:
            raise forms.ValidationError(
                {"symptoms_g4": "Invalid selection. Must be from above list of symptoms"}
            )

        # provide detail always
        self.m2m_other_not_specify(
            NONE, m2m_field="symptoms_g3", field_other="symptoms_g3_detail"
        )
        # there should not be overlap between the G3 and G4 lists
        if (
            symptoms_g4 != [NONE]
            and OTHER not in symptoms_g4
            and [x for x in symptoms_g4 if x in symptoms_g3]
        ):
            raise forms.ValidationError(
                {"symptoms_g4": "Invalid selection. G3 and G4 events must be unique"}
            )
        self.m2m_other_not_specify(
            NONE, m2m_field="symptoms_g4", field_other="symptoms_g4_detail"
        )
        if (
            symptoms_g3 != [NONE]
            and OTHER not in symptoms_g3
            and [x for x in symptoms_g3 if x in symptoms_g4]
        ):
            raise forms.ValidationError(
                {"symptoms_g3": "Invalid selection. G3 and G4 events must be unique"}
            )

    def validate_other_problems(self):
        self.required_if(
            YES, field="any_other_problems", field_required="any_other_problems_detail"
        )
        self.applicable_if(
            YES,
            field="any_other_problems",
            field_applicable="any_other_problems_sae",
        )
        self.applicable_if(
            YES,
            field="any_other_problems_sae",
            field_applicable="any_other_problems_sae_grade",
        )

    def validate_hospitalizations(self):
        self.required_if(YES, field="attend_clinic", field_required="admitted_hospital")
        self.required_if(YES, field="attend_clinic", field_required="attend_clinic_details")
        self.required_if(YES, field="attend_clinic", field_required="attend_clinic_sae")
        self.required_if(YES, field="attend_clinic", field_required="prescribed_medication")
        self.required_if(
            YES,
            field="prescribed_medication",
            field_required="prescribed_medication_detail",
        )

    def validate_hiv_medications(self):
        self.required_if(YES, field="art_change", field_required="art_change_reason")
        self.required_if(YES, field="art_change", field_required="art_new_regimen")
        self.validate_other_specify(
            field="art_new_regimen", other_specify_field="art_new_regimen_other"
        )


class FollowupExaminationForm(CrfModelFormMixin, ActionItemCrfFormMixin, forms.ModelForm):

    form_validator_cls = FollowupExaminationFormValidator

    class Meta(ActionItemCrfFormMixin.Meta):
        model = FollowupExamination
        fields = "__all__"



# Generated by Django 4.0.4 on 2022-05-12 23:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_screening", "0043_auto_20220407_1713"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="severe_htn",
            field=models.CharField(
                blank=True,
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="Based on the above readings. Severe HTN is any BP reading > 180/110mmHg",
                max_length=15,
                null=True,
                verbose_name="Does the patient have severe hypertension?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="vl_undetectable",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                max_length=15,
                verbose_name="Does the patient have a viral load measure of less than 1000 copies per ml taken <u>within the last 12 months</u>",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="severe_htn",
            field=models.CharField(
                blank=True,
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="Based on the above readings. Severe HTN is any BP reading > 180/110mmHg",
                max_length=15,
                null=True,
                verbose_name="Does the patient have severe hypertension?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="vl_undetectable",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                max_length=15,
                verbose_name="Does the patient have a viral load measure of less than 1000 copies per ml taken <u>within the last 12 months</u>",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="severe_htn",
            field=models.CharField(
                blank=True,
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="Based on the above readings. Severe HTN is any BP reading > 180/110mmHg",
                max_length=15,
                null=True,
                verbose_name="Does the patient have severe hypertension?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="vl_undetectable",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                max_length=15,
                verbose_name="Does the patient have a viral load measure of less than 1000 copies per ml taken <u>within the last 12 months</u>",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="severe_htn",
            field=models.CharField(
                blank=True,
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="Based on the above readings. Severe HTN is any BP reading > 180/110mmHg",
                max_length=15,
                null=True,
                verbose_name="Does the patient have severe hypertension?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="vl_undetectable",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                max_length=15,
                verbose_name="Does the patient have a viral load measure of less than 1000 copies per ml taken <u>within the last 12 months</u>",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="severe_htn",
            field=models.CharField(
                blank=True,
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="Based on the above readings. Severe HTN is any BP reading > 180/110mmHg",
                max_length=15,
                null=True,
                verbose_name="Does the patient have severe hypertension?",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="vl_undetectable",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                max_length=15,
                verbose_name="Does the patient have a viral load measure of less than 1000 copies per ml taken <u>within the last 12 months</u>",
            ),
        ),
    ]



import os  # noqa

from multisite import SiteID

from .defaults import *  # noqa

print(f"Settings file {__file__}")  # noqa

SITE_ID = SiteID(default=30)
EDC_SITES_UAT_DOMAIN = False
DEBUG = True
ALLOWED_HOSTS = [
    "localhost",
    "mnazi-moja.tz.meta3.clinicedc.org",
    "mbagala.tz.meta3.clinicedc.org",
    "mwananyamala.tz.meta3.clinicedc.org",
    "hindu-mandal.tz.meta3.clinicedc.org",
    "temeke.tz.meta3.clinicedc.org",
    "amana.tz.meta3.clinicedc.org",
]

SECURE_SSL_REDIRECT = False

if os.path.exists(BASE_DIR) and not os.path.exists(KEY_PATH):  # noqa
    os.makedirs(KEY_PATH)  # noqa
    AUTO_CREATE_KEYS = True



from edc_model.models import BaseUuidModel

from ..model_mixins import CrfModelMixin


class ConcomitantMedication(CrfModelMixin, BaseUuidModel):

    # prescription
    # days
    # dosage
    # instructions

    class Meta(CrfModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "Concomitant Medication"
        verbose_name_plural = "Concomitant Medication"



from edc_action_item.auth_objects import ACTION_ITEM, ACTION_ITEM_EXPORT
from edc_adverse_event.auth_objects import TMG_ROLE
from edc_appointment.auth_objects import APPOINTMENT_EXPORT
from edc_auth.auth_objects import (
    AUDITOR,
    AUDITOR_ROLE,
    CLINIC,
    CLINIC_SUPER,
    CLINICIAN_ROLE,
    CLINICIAN_SUPER_ROLE,
    NURSE_ROLE,
    PII,
)
from edc_auth.site_auths import site_auths
from edc_data_manager.auth_objects import DATA_MANAGER_EXPORT, DATA_MANAGER_ROLE
from edc_export.auth_objects import DATA_EXPORTER_ROLE
from edc_mnsi.auth_objects import MNSI, MNSI_SUPER, MNSI_VIEW
from edc_pharmacy.auth_objects import (
    PHARMACIST_ROLE,
    PHARMACY_PRESCRIBER,
    SITE_PHARMACIST_ROLE,
)
from edc_qol.auth_objects import QOL, QOL_SUPER, QOL_VIEW
from edc_randomization.auth_objects import RANDO_BLINDED, RANDO_UNBLINDED
from edc_screening.auth_objects import SCREENING, SCREENING_SUPER, SCREENING_VIEW
from edc_subject_dashboard.auths import SUBJECT_VIEW
from edc_unblinding.auth_objects import UNBLINDING_REQUESTORS

from .auth_objects import clinic_codenames, screening_codenames

# update edc_auth default groups
site_auths.update_group(*clinic_codenames, name=AUDITOR, view_only=True)
site_auths.update_group(*clinic_codenames, name=CLINIC, no_delete=True)
site_auths.update_group(*clinic_codenames, name=CLINIC_SUPER)
site_auths.update_group(*screening_codenames, name=SCREENING, no_delete=True)
site_auths.update_group(*screening_codenames, name=SCREENING_SUPER)
site_auths.update_group(*screening_codenames, name=SCREENING_VIEW, view_only=True)

# update edc_auth default roles
site_auths.update_role(
    UNBLINDING_REQUESTORS,
    MNSI,
    QOL,
    PHARMACY_PRESCRIBER,
    name=CLINICIAN_ROLE,
)
site_auths.update_role(
    MNSI_SUPER,
    QOL_SUPER,
    UNBLINDING_REQUESTORS,
    PHARMACY_PRESCRIBER,
    name=CLINICIAN_SUPER_ROLE,
)
site_auths.update_role(MNSI, QOL, name=NURSE_ROLE)
site_auths.update_role(MNSI_VIEW, QOL_VIEW, name=AUDITOR_ROLE)
site_auths.update_role(
    ACTION_ITEM_EXPORT,
    APPOINTMENT_EXPORT,
    DATA_MANAGER_EXPORT,
    name=DATA_EXPORTER_ROLE,
)
site_auths.update_role(RANDO_UNBLINDED, PII, name=PHARMACIST_ROLE)
site_auths.update_role(RANDO_BLINDED, PII, name=SITE_PHARMACIST_ROLE)
site_auths.update_role(SUBJECT_VIEW, SCREENING_VIEW, name=DATA_MANAGER_ROLE)
site_auths.update_role(
    SUBJECT_VIEW, SCREENING_VIEW, ACTION_ITEM, UNBLINDING_REQUESTORS, name=TMG_ROLE
)



from django.test import TestCase
from edc_constants.constants import COMPLETE, NO, NONE, NOT_APPLICABLE, YES
from edc_list_data import PreloadData

from meta_lists.models import (
    ArvRegimens,
    BaselineSymptoms,
    DiabetesSymptoms,
    OiProphylaxis,
)
from meta_screening.tests.meta_test_case_mixin import MetaTestCaseMixin
from meta_screening.tests.options import now
from meta_subject.forms import PatientHistoryForm


class BaseTestPatientHistory(MetaTestCaseMixin, TestCase):
    def get_options(self):
        self.subject_visit = self.get_subject_visit(appt_datetime=now)
        symptoms = BaselineSymptoms.objects.filter(name=NONE)
        arv_regimen = ArvRegimens.objects.filter(name="TDF_3TC_ATV_r")
        oi_prophylaxis = OiProphylaxis.objects.filter(name__in=["fluconazole", "isoniazid"])
        dm_symptoms = DiabetesSymptoms.objects.all()
        return {
            "current_arv_regimen": arv_regimen[0].id,
            "current_smoker": YES,
            "dia_blood_pressure": 80,
            "dm_in_family": NO,
            "dm_symptoms": dm_symptoms,
            "dyslipidaemia_diagnosis": NO,
            "on_dyslipidaemia_treatment": NOT_APPLICABLE,
            "dyslipidaemia_rx": NOT_APPLICABLE,
            "family_diabetics": NO,
            "former_smoker": NOT_APPLICABLE,
            "has_abdominal_tenderness": NO,
            "has_enlarged_liver": NO,
            "has_previous_arv_regimen": NO,
            "heart_rate": 65,
            "htn_diagnosis": NO,
            "htn_treatment": None,
            "is_heartbeat_regular": YES,
            "jaundice": YES,
            "oi_prophylaxis": oi_prophylaxis,
            "on_htn_treatment": NO,
            "on_oi_prophylaxis": YES,
            "other_dm_symptoms": "erik",
            "past_year_symptoms": None,
            "peripheral_oedema": YES,
            "previous_arv_regimen": [],
            "report_datetime": self.subject_visit.report_datetime,
            "respiratory_rate": 12,
            "subject_visit": self.subject_visit.pk,
            "symptoms": symptoms,
            "sys_blood_pressure": 120,
            "taking_statins": YES,
            "temperature": 37,
            "waist_circumference": 61,
            "weight": 65,
            "crf_status": COMPLETE,
            "vl_undetectable": YES,
        }


class TestPatientHistoryPhaseThree(BaseTestPatientHistory):
    def test_ok_phase_three(self):
        from meta_lists.list_data import list_data

        PreloadData(list_data=list_data)
        data = {k: v for k, v in self.get_options().items()}
        form = PatientHistoryForm(data=data)
        form.is_valid()
        self.assertEqual(form._errors, {})



from django.contrib import admin
from edc_adherence.model_admin_mixin import MedicationAdherenceAdminMixin

from ..admin_site import meta_subject_admin
from ..forms import MedicationAdherenceForm
from ..models import MedicationAdherence
from .modeladmin import CrfModelAdmin

pill_count_fieldset_tuple = (
    "Pill Count",
    {
        "fields": ["pill_count_performed", "pill_count_not_performed_reason", "pill_count"],
    },
)


@admin.register(MedicationAdherence, site=meta_subject_admin)
class MedicationAdherenceAdmin(MedicationAdherenceAdminMixin, CrfModelAdmin):

    form = MedicationAdherenceForm



# Generated by Django 3.2.4 on 2021-07-02 01:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_screening", "0021_auto_20210628_2105"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="ifg_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                help_text="A `HIGH` reading may be entered as 9999.99",
                max_digits=8,
                null=True,
                verbose_name="IFG level",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="ifg_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                help_text="A `HIGH` reading may be entered as 9999.99",
                max_digits=8,
                null=True,
                verbose_name="IFG level",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="ifg_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                help_text="A `HIGH` reading may be entered as 9999.99",
                max_digits=8,
                null=True,
                verbose_name="IFG level",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="ifg_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                help_text="A `HIGH` reading may be entered as 9999.99",
                max_digits=8,
                null=True,
                verbose_name="IFG level",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="ifg_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                help_text="A `HIGH` reading may be entered as 9999.99",
                max_digits=8,
                null=True,
                verbose_name="IFG level",
            ),
        ),
    ]



from django import forms
from edc_form_validators import FormValidatorMixin
from edc_screening.modelform_mixins import AlreadyConsentedFormMixin

from ..form_validators import ScreeningPartOneFormValidator
from ..models import ScreeningPartOne
from .field_lists import part_one_fields


class ScreeningPartOneForm(AlreadyConsentedFormMixin, FormValidatorMixin, forms.ModelForm):

    form_validator_cls = ScreeningPartOneFormValidator

    class Meta:
        model = ScreeningPartOne
        fields = part_one_fields



# Generated by Django 2.2.6 on 2019-11-07 19:49

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_screening", "0014_auto_20191107_0528"),
    ]

    operations = [
        migrations.AddField(
            model_name="historicalscreeningpartone",
            name="refused",
            field=models.BooleanField(default=False, editable=False),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartthree",
            name="refused",
            field=models.BooleanField(default=False, editable=False),
        ),
        migrations.AddField(
            model_name="historicalscreeningparttwo",
            name="refused",
            field=models.BooleanField(default=False, editable=False),
        ),
        migrations.AddField(
            model_name="historicalsubjectscreening",
            name="refused",
            field=models.BooleanField(default=False, editable=False),
        ),
        migrations.AddField(
            model_name="subjectscreening",
            name="refused",
            field=models.BooleanField(default=False, editable=False),
        ),
    ]



# Generated by Django 3.2.7 on 2021-10-03 14:09

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("meta_prn", "0013_auto_20210911_2036"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="historicallosstofollowup",
            options={
                "get_latest_by": "history_date",
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Loss to Follow Up",
            },
        ),
        migrations.AlterModelOptions(
            name="losstofollowup",
            options={
                "verbose_name": "Loss to Follow Up",
                "verbose_name_plural": "Loss to Follow Up",
            },
        ),
    ]



from django.db import models
from edc_model.models import BaseUuidModel
from edc_visit_tracking.model_mixins import SubjectVisitMissedModelMixin

from meta_lists.models import SubjectVisitMissedReasons

from ..constants import MISSED_VISIT_ACTION
from ..model_mixins import CrfWithActionModelMixin


class SubjectVisitMissed(
    SubjectVisitMissedModelMixin,
    CrfWithActionModelMixin,
    BaseUuidModel,
):

    action_name = MISSED_VISIT_ACTION

    missed_reasons = models.ManyToManyField(
        SubjectVisitMissedReasons, blank=True, related_name="meta_missed_reasons"
    )

    class Meta(CrfWithActionModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "Missed Visit Report"
        verbose_name_plural = "Missed Visit Report"



from django.contrib import admin
from django_audit_fields.admin import audit_fieldset_tuple
from edc_crf.fieldset import crf_status_fieldset

from ..admin_site import meta_subject_admin
from ..forms import HepatitisTestForm
from ..models import HepatitisTest
from .modeladmin import CrfModelAdmin


@admin.register(HepatitisTest, site=meta_subject_admin)
class HepatitisTestAdmin(CrfModelAdmin):

    form = HepatitisTestForm

    fieldsets = (
        (
            None,
            {
                "fields": (
                    "subject_visit",
                    "report_datetime",
                )
            },
        ),
        (
            "Hepatitis B",
            {
                "fields": (
                    "hbsag_performed",
                    "hbsag",
                    "hbsag_date",
                )
            },
        ),
        (
            "Hepatitis C",
            {
                "fields": (
                    "hcv_performed",
                    "hcv",
                    "hcv_date",
                )
            },
        ),
        crf_status_fieldset,
        audit_fieldset_tuple,
    )

    radio_fields = {
        "hbsag_performed": admin.VERTICAL,
        "hbsag": admin.VERTICAL,
        "hcv_performed": admin.VERTICAL,
        "hcv": admin.VERTICAL,
    }



from edc_lab.model_mixins import CrfWithRequisitionModelMixin
from edc_lab_panel.panels import hba1c_panel
from edc_lab_results import BLOOD_RESULTS_HBA1C_ACTION
from edc_lab_results.model_mixins import BloodResultsModelMixin, Hba1cModelMixin
from edc_model.models import BaseUuidModel

from ...model_mixins import CrfWithActionModelMixin


class BloodResultsHba1c(
    CrfWithActionModelMixin,
    Hba1cModelMixin,
    CrfWithRequisitionModelMixin,
    BloodResultsModelMixin,
    BaseUuidModel,
):
    action_name = BLOOD_RESULTS_HBA1C_ACTION
    tracking_identifier_prefix = "HA"
    lab_panel = hba1c_panel

    class Meta(CrfWithActionModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "Blood Result: HbA1c"
        verbose_name_plural = "Blood Results: HbA1c"



from copy import deepcopy

from dateutil.relativedelta import relativedelta
from django.test import TestCase
from edc_constants.constants import NO, NOT_APPLICABLE, TBD, YES
from edc_utils.date import get_utcnow

from meta_pharmacy.constants import METFORMIN
from meta_screening.models import ScreeningPartOne, ScreeningPartTwo

from ...eligibility import EligibilityPartOne
from ..options import get_part_one_eligible_options, get_part_two_eligible_options


class TestScreeningPartTwo(TestCase):
    def setUp(self):
        part_one_eligible_options = deepcopy(get_part_one_eligible_options())
        model_obj = ScreeningPartOne(**part_one_eligible_options)
        model_obj.save()
        self.assertIsNone(getattr(model_obj, EligibilityPartOne.reasons_ineligible_fld_name))
        self.assertEqual(
            getattr(model_obj, EligibilityPartOne.eligible_fld_name),
            EligibilityPartOne.is_eligible_value,
        )

        self.screening_identifier = model_obj.screening_identifier

    def test_defaults_phase_three(self):
        self._test_defaults()

    def test_eligible_phase_three(self):
        self._test_eligible()

    def _test_defaults(self):

        obj = ScreeningPartTwo.objects.get(screening_identifier=self.screening_identifier)
        self.assertEqual(obj.eligible_part_two, TBD)
        self.assertIn("not answered", obj.reasons_ineligible_part_two)

        self.assertFalse(obj.eligible)
        self.assertFalse(obj.consented)

    def _test_eligible(self):

        obj = ScreeningPartTwo.objects.get(screening_identifier=self.screening_identifier)
        self.assertEqual(obj.eligible_part_one, YES)
        self.assertIsNone(obj.reasons_ineligible_part_one)

        part_two_eligible_options = deepcopy(get_part_two_eligible_options())

        for k, v in part_two_eligible_options.items():
            setattr(obj, k, v)

        obj.advised_to_fast = NOT_APPLICABLE
        obj.appt_datetime = None
        obj.acute_condition = None
        obj.metformin_sensitivity = None
        obj.save()
        obj.refresh_from_db()

        self.assertIn("not answered", obj.reasons_ineligible_part_two)
        self.assertNotIn("Appt Datetime", obj.reasons_ineligible_part_two)
        self.assertNotIn("Advised To Fast", obj.reasons_ineligible_part_two)
        self.assertEqual(obj.eligible_part_two, TBD)
        self.assertFalse(obj.eligible)
        self.assertFalse(obj.consented)

        obj.acute_condition = NO
        obj.metformin_sensitivity = YES
        obj.save()

        self.assertIn(METFORMIN, (obj.reasons_ineligible_part_two or "").lower())
        self.assertEqual(obj.eligible_part_two, NO)
        self.assertFalse(obj.eligible)
        self.assertFalse(obj.consented)

        obj.acute_condition = NO
        obj.metformin_sensitivity = NO
        obj.advised_to_fast = YES
        obj.appt_datetime = get_utcnow() + relativedelta(days=1)
        obj.save()

        self.assertIsNone(obj.reasons_ineligible_part_two)
        self.assertEqual(obj.eligible_part_two, YES)
        self.assertFalse(obj.eligible)
        self.assertFalse(obj.consented)



# Generated by Django 4.0.4 on 2022-05-12 23:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("edc_mnsi", "0003_alter_mnsi_abnormal_obs_left_foot_and_more"),
        ("meta_subject", "0110_auto_20220512_1811"),
    ]

    operations = [
        migrations.AlterField(
            model_name="followupvitals",
            name="severe_htn",
            field=models.CharField(
                blank=True,
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="Based on the above readings. Severe HTN is any BP reading > 180/110mmHg",
                max_length=15,
                null=True,
                verbose_name="Does the patient have severe hypertension?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalfollowupvitals",
            name="severe_htn",
            field=models.CharField(
                blank=True,
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="Based on the above readings. Severe HTN is any BP reading > 180/110mmHg",
                max_length=15,
                null=True,
                verbose_name="Does the patient have severe hypertension?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalphysicalexam",
            name="severe_htn",
            field=models.CharField(
                blank=True,
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="Based on the above readings. Severe HTN is any BP reading > 180/110mmHg",
                max_length=15,
                null=True,
                verbose_name="Does the patient have severe hypertension?",
            ),
        ),
        migrations.AlterField(
            model_name="mnsi",
            name="abnormal_obs_left_foot",
            field=models.ManyToManyField(
                blank=True,
                related_name="+",
                to="edc_mnsi.abnormalfootappearanceobservations",
                verbose_name="If NO, check all that apply to LEFT foot?",
            ),
        ),
        migrations.AlterField(
            model_name="mnsi",
            name="abnormal_obs_right_foot",
            field=models.ManyToManyField(
                blank=True,
                related_name="+",
                to="edc_mnsi.abnormalfootappearanceobservations",
                verbose_name="If NO, check all that apply to RIGHT foot?",
            ),
        ),
        migrations.AlterField(
            model_name="physicalexam",
            name="severe_htn",
            field=models.CharField(
                blank=True,
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="Based on the above readings. Severe HTN is any BP reading > 180/110mmHg",
                max_length=15,
                null=True,
                verbose_name="Does the patient have severe hypertension?",
            ),
        ),
    ]



from edc_constants.constants import NEG, NO, NOT_APPLICABLE
from edc_egfr import EgfrCkdEpi
from edc_egfr.calculators import EgfrCalculatorError
from edc_reportable import (
    MICROMOLES_PER_LITER,
    MILLIMOLES_PER_LITER,
    ConversionNotHandled,
    calculate_bmi,
    convert_units,
)
from edc_screening.screening_eligibility import FC, ScreeningEligibility


class BaseEligibilityPartThree(ScreeningEligibility):
    eligible_fld_name = "eligible_part_three"
    reasons_ineligible_fld_name = "reasons_ineligible_part_three"

    def __init__(self, **kwargs):
        self.bmi = None
        self.calculated_egfr_value = None
        self.creatinine_units = None
        self.creatinine_value = None
        self.height = None
        self.fbg_units = None
        self.fbg2_units = None
        self.fbg_value = None
        self.fbg2_value = None
        self.ogtt_units = None
        self.ogtt_value = None
        self.ogtt2_units = None
        self.ogtt2_value = None
        self.repeat_glucose_opinion = None
        self.repeat_glucose_performed = None
        self.weight = None
        self.unsuitable_agreed = None
        self.urine_bhcg_value = None
        super().__init__(**kwargs)

    def assess_eligibility(self) -> None:
        if self.weight and self.height:
            self.bmi = calculate_bmi(weight_kg=self.weight, height_cm=self.height)
        try:
            self.calculated_egfr_value = EgfrCkdEpi(
                gender=self.model_obj.gender,
                age_in_years=self.model_obj.age_in_years,
                ethnicity=self.model_obj.gender,
                creatinine_value=self.model_obj.creatinine_value,
                creatinine_units=self.model_obj.creatinine_units,
            ).value
        except EgfrCalculatorError:
            pass

    def set_fld_attrs_on_model(self) -> None:
        self.model_obj.converted_creatinine_value = self.converted_creatinine_value
        self.model_obj.converted_fbg_value = self.converted_fbg_value
        self.model_obj.converted_ogtt_value = self.converted_ogtt_value
        self.model_obj.converted_fbg2_value = self.converted_fbg2_value
        self.model_obj.converted_ogtt2_value = self.converted_ogtt2_value
        if self.bmi:
            self.model_obj.calculated_bmi_value = self.bmi.value

    def get_required_fields(self) -> dict[str, FC]:
        return {
            "creatinine_units": FC(ignore_if_missing=True),
            "creatinine_value": FC(ignore_if_missing=True),
            "height": FC(value=range(0, 500), msg="Missing height"),
            "fbg_units": FC(ignore_if_missing=True),
            "fbg2_units": FC(ignore_if_missing=True),
            "fbg_value": FC(ignore_if_missing=True),
            "fbg2_value": FC(ignore_if_missing=True),
            "ogtt_units": FC(ignore_if_missing=True),
            "ogtt_value": FC(ignore_if_missing=True),
            "ogtt2_units": FC(ignore_if_missing=True),
            "ogtt2_value": FC(ignore_if_missing=True),
            "weight": FC(value=range(0, 500), msg="Missing weight"),
            "repeat_glucose_opinion": FC(ignore_if_missing=True),
            "repeat_glucose_performed": FC(ignore_if_missing=True),
            "urine_bhcg_value": FC(value=[NEG, NOT_APPLICABLE], msg="UPT positive"),
            "unsuitable_agreed": FC(value=[NO, NOT_APPLICABLE]),
        }

    @property
    def converted_creatinine_value(self):
        try:
            value = convert_units(
                self.creatinine_value,
                units_from=self.creatinine_units,
                units_to=MICROMOLES_PER_LITER,
            )
        except ConversionNotHandled as e:
            raise ConversionNotHandled(f"Creatinine. {e}")
        return value

    @property
    def converted_fbg_value(self):
        try:
            value = convert_units(
                self.fbg_value,
                units_from=self.fbg_units,
                units_to=MILLIMOLES_PER_LITER,
            )
        except ConversionNotHandled as e:
            raise ConversionNotHandled(f"FBG. {e}")
        return value

    @property
    def converted_fbg2_value(self):
        try:
            value = convert_units(
                self.fbg2_value,
                units_from=self.fbg2_units,
                units_to=MILLIMOLES_PER_LITER,
            )
        except ConversionNotHandled as e:
            raise ConversionNotHandled(f"FBG2. {e}")
        return value

    @property
    def converted_ogtt_value(self):
        try:
            value = convert_units(
                self.ogtt_value,
                units_from=self.ogtt_units,
                units_to=MILLIMOLES_PER_LITER,
            )
        except ConversionNotHandled as e:
            raise ConversionNotHandled(f"OGTT. {e}")
        return value

    @property
    def converted_ogtt2_value(self):
        try:
            value = convert_units(
                self.ogtt2_value,
                units_from=self.ogtt2_units,
                units_to=MILLIMOLES_PER_LITER,
            )
        except ConversionNotHandled as e:
            raise ConversionNotHandled(f"OGTT2. {e}")
        return value

    def set_eligible_model_field(self):
        setattr(self.model_obj, self.eligible_fld_name, self.eligible)



from .ae_followup_form import AeFollowupForm
from .ae_initial_form import AeInitialForm
from .ae_local_review_form import AeLocalReviewForm
from .ae_sponsor_review_form import AeSponsorReviewForm
from .ae_susar_form import AeSusarForm
from .ae_tmg_form import AeTmgForm
from .death_report_form import DeathReportForm
from .death_report_tmg_form import DeathReportTmgForm
from .death_report_tmg_second_form import DeathReportTmgSecondForm



from django import forms
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_form_validators import INVALID_ERROR
from edc_pharmacy.form_validators import (
    StudyMedicationFormValidator as BaseStudyMedicationFormValidator,
)
from edc_pharmacy.forms import StudyMedicationModelFormMixin
from edc_visit_schedule.constants import DAY1

from ..models import StudyMedication


class StudyMedicationFormValidator(BaseStudyMedicationFormValidator):
    def clean(self):
        super().clean()
        self.validate_half_dose_at_baseline()

    def validate_half_dose_at_baseline(self):
        """Require 1000mg dose at baseline"""
        try:
            subject_visit = (
                self.cleaned_data.get("subject_visit") or self.instance.related_visit
            )
        except AttributeError:
            self.raise_validation_error("Subject visit is required", INVALID_ERROR)
        else:
            if subject_visit.visit_code == DAY1 and subject_visit.visit_code_sequence == 0:
                if (
                    self.cleaned_data.get("dosage_guideline")
                    and self.cleaned_data.get("dosage_guideline").dose != 1000
                ):
                    raise forms.ValidationError(
                        {"dosage_guideline": "Invalid. Expected 1000mg/day at baseline"}
                    )


class StudyMedicationForm(StudyMedicationModelFormMixin, CrfModelFormMixin, forms.ModelForm):

    form_validator_cls = StudyMedicationFormValidator

    class Meta:
        model = StudyMedication
        fields = "__all__"



# Generated by Django 3.2.13 on 2022-07-14 20:03

from django.db import migrations
from edc_lab_panel.panels import (
    blood_glucose_panel,
    fbc_panel,
    hba1c_panel,
    insulin_panel,
    lft_panel,
    lipids_panel,
    rft_panel,
)
from edc_lab_results.calculate_missing import calculate_missing
from tqdm import tqdm


def update_bloodresults(apps, schema_editor):
    app_label = "meta_subject"
    for suffix, panel in {
        "fbc": fbc_panel,
        "glu": blood_glucose_panel,
        "hba1c": hba1c_panel,
        "ins": insulin_panel,
        "lft": lft_panel,
        "lipid": lipids_panel,
        "rft": rft_panel,
    }.items():
        model_cls = apps.get_model(f"{app_label}.bloodresults{suffix}")
        cnt = model_cls.objects.all().count()
        print(model_cls._meta.label_lower)
        for obj in tqdm(model_cls.objects.all(), total=cnt):
            obj.missing_count, obj.missing = calculate_missing(obj, panel)
            obj.save_base(update_fields=["missing_count", "missing"])


class Migration(migrations.Migration):

    dependencies = [
        ("meta_subject", "0123_auto_20220714_2136"),
    ]

    operations = [migrations.RunPython(update_bloodresults)]
